@@ -126,6 +126,7 @@ summarize_sir_results <- function(sir_df,
   
   #prepare site_var_name
   
+  site_grouped <- FALSE
   if(!is.character(site_var_name)){
     rlang::warn("Parameter `site_var_name` must be character vector. Default `site_var_name = \"t_site\"` will be used instead.")
     site_var_name <- "t_site"
@@ -135,6 +136,14 @@ summarize_sir_results <- function(sir_df,
     }else{
       if((site_var_name %in% colnames(sir_df))){
         cs <- TRUE
+        #add additional check if a different variable than t_site is used
+        if("t_site" %in% colnames(sir_df)){
+          rlang::warn(paste0(
+            "The provided `site_var_name == `", site_var_name,  " is different from default `site_var_name = \"t_site\"`.\n", 
+            "We assume, you want to group results by a different site variable than used when using the `sir_by_futime()` function. \n",
+            "We try to adjust calculation of Observed, PYARs and SIR accordingly, but make sure that you check results for correctness!"))
+          site_grouped <- TRUE
+        }
       }else{
         rlang::warn("Provided `site_var_name` does not exit in sir_df. Default `site_var_name = \"t_site\"` will be used instead.")
         site_var_name <- "t_site"
@@ -207,9 +216,15 @@ summarize_sir_results <- function(sir_df,
   #prepare site_var_name
   #in case t_site var need to be changed
   if(cs){
-    sir_df <- sir_df %>%
-      tidytable::select.(-t_site) %>%
-      tidytable::rename.(t_site = !!rlang::sym(site_var_name))
+    if(site_grouped == FALSE){
+      sir_df <- sir_df %>%
+        tidytable::select.(-t_site) %>%
+        tidytable::rename.(t_site = !!rlang::sym(site_var_name))
+    }else{
+      sir_df <- sir_df %>%
+        tidytable::rename.(t_site_orig = t_site,
+                           t_site = !!rlang::sym(site_var_name))
+    }
   }
   
   
@@ -260,13 +275,27 @@ summarize_sir_results <- function(sir_df,
   
   
   #enforce summarize site
-  #BUG? Check whether this implementation of t_site is really okay (count too many pyar?)
   
   if(summarize_site == TRUE){
     sg <- TRUE
     sg_var_names <- rlang::eval_tidy(c(summarize_groups, "t_site"))
   }
   
+  # if(summarize_site == TRUE){
+  #   if(site_grouped == FALSE){
+  #   sg <- TRUE
+  #   sg_var_names <- rlang::eval_tidy(c(summarize_groups, "t_site"))
+  #   } else{
+  #     #FIX: check whether this safeguard here is really needed.
+  #     rlang::warn(paste0(
+  #     "Parameter `summarize_site == TRUE` provided, but function detected that you are not using the original `site_var_name == t_site`. \n",
+  #     "We therefore assume you are using a grouped site_var for which summarize_site makes no sense. \n",
+  #     "Default `summarize_site = FALSE` will be used instead."))
+  #     summarize_site <- FALSE
+  #     sg <- FALSE
+  #   }
+  # }
+  
   #prepare total_fu
   
   if(add_total_fu == "start" | add_total_fu == "end" | add_total_fu == "only"){
@@ -400,12 +429,18 @@ summarize_sir_results <- function(sir_df,
     
     #iii) remove from grouping vars those who should be summarized
     grouping_vars <- all_grouping_vars[!(all_grouping_vars %in% sg_var_names)]
-    
+    if(site_grouped == TRUE){
+      grouping_vars_with_site <- c(grouping_vars, "t_site_orig")
+    } else{
+      grouping_vars_with_site <- c(grouping_vars, "t_site")
+    }
     
     #iv) summarize over grouping vars
-    sum_pre_tmp <- sir_df %>%
+    
+    #first all variables that are varying across t_site
+    sum_pre_tmp_a <- sir_df %>%
       tidytable::summarize_across.(
-        .cols = c(observed, pyar, n_base, ref_inc_cases, ref_population_pyar, expected),
+        .cols = c(observed, expected, ref_inc_cases),
         .fns =  ~ sum(.x, na.rm = TRUE),
         .names = "group_{.col}",
         .by = !!grouping_vars) %>%
@@ -414,10 +449,67 @@ summarize_sir_results <- function(sir_df,
         sir = .SD$group_observed / .SD$group_expected,
         sir_lci = (stats::qchisq(p = alpha / 2, df = 2 * .SD$group_observed) / 2) / .SD$group_expected,
         sir_uci = (stats::qchisq(p = 1 - alpha / 2, df = 2 * (.SD$group_observed + 1)) / 2) / .SD$group_expected,
-        group_incidence_crude_rate = .SD$group_ref_inc_cases / .SD$group_ref_population_pyar * 100000
       ) %>%
       tidytable::distinct.()
     
+    #second, variables that are constant over t_site
+    sum_pre_tmp_b <- sir_df %>%
+      tidytable::summarize_across.(
+        .cols = c(pyar, n_base, ref_population_pyar),
+        .fns =  ~ round(sum(.x, na.rm = TRUE), 2),
+        .names = "group_{.col}",
+        .by = !!grouping_vars_with_site) %>% 
+      tidytable::select.(-tidyselect::any_of("t_site_orig")) %>%
+      tidytable::distinct.(tidyselect::all_of(c(grouping_vars, "group_pyar", "group_n_base", "group_ref_population_pyar")), .keep_all = TRUE)
+    
+    #in some instances, there will be ambiguous results for n_base and ref_population_pyar
+    #created by the fact that sir_results is filtered to exclude non informative strata 
+    #(i.e. no case occured in the reference population and no FU time contributed in the dataset).
+    #This occurs for strata where cases are observed that don't occur in refrates_df. --> see sir_df$warning)
+    #therefore we include a check
+    
+    if(nrow(sum_pre_tmp_b) != nrow(sum_pre_tmp_a)){
+      #give warning
+      rlang::warn(
+        paste0(
+          "The results file `sir_df` contains observed cases in i_observed that do not occur in the refrates_df (ref_inc_cases).\n",
+          "Therefore calculation of the variables n_base and ref_population_pyar is ambiguous. \n",
+          "We take the first value of each variable. Expect small inconsistencies in the calculation of n_base, ref_population_pyar and ref_inc_crude_rate across strata. \n",
+          "If you want to know more, please check the `warnings` column of `sir_df`.")
+      )
+      #take first results only
+      sum_pre_tmp_b <- sum_pre_tmp_b %>%
+        tidytable::distinct.(tidyselect::all_of(c(grouping_vars, "group_pyar")), .keep_all = TRUE)
+    }
+    
+    #check that merge will work
+    if(nrow(sum_pre_tmp_b) != nrow(sum_pre_tmp_a)){
+      #give warning
+      rlang::abort(
+        paste0(
+          "Merge error. \n",
+          "sum_pre_tmp_a and sum_pre_tmp_b have unequal numbers of rows."
+        )
+      )
+    }
+    
+    #now merge a and b
+    sum_pre_tmp <- sum_pre_tmp_a %>%
+      tidytable::left_join.(sum_pre_tmp_b, by = grouping_vars) %>%
+      #calculate crude rate
+      tidytable::mutate.(
+        group_incidence_crude_rate = .SD$group_ref_inc_cases / .SD$group_ref_population_pyar * 100000
+      ) %>%
+      #ensure same sorting as before
+      tidytable::select.(tidyselect::all_of(
+        c(grouping_vars, 
+          "group_observed", "group_expected", "sir", "sir_lci", "sir_uci",
+          "group_pyar", "group_n_base", "group_ref_inc_cases", "group_ref_population_pyar", "group_incidence_crude_rate")
+      ))
+    
+    rm(sum_pre_tmp_a, sum_pre_tmp_b)
+    
+    
     
     #v) #add grouping information for summarized variables
     
@@ -501,12 +593,14 @@ summarize_sir_results <- function(sir_df,
       tidytable::rename.(observed = group_observed,
                          expected = group_expected,
                          pyar = group_pyar,
+                         n_base = group_n_base,
                          ref_inc_cases = group_ref_inc_cases,
                          ref_population_pyar = group_ref_population_pyar,
                          ref_inc_crude_rate = group_incidence_crude_rate) %>%
       tidytable::select.(tidyselect::any_of(c("age", "region", "sex", "year", if(rs){"race"},
-                                              if(yb){c("yvar_name", "yvar_label")}, if(xb){c("xvar_name", "xvar_label")}, 
-                                              if(fu){"fu_time"}, 
+                                              if(yb){c("yvar_name", "yvar_label", "yvar_sort", "yvar_sort_levels")}, 
+                                              if(xb){c("xvar_name", "xvar_label")}, 
+                                              if(fu){c("fu_time", "fu_time_sort")}, 
                                               "t_site", "observed", "expected", "sir",
                                               if(collapse_ci == TRUE){"sir_ci"},
                                               if(collapse_ci == FALSE){c("sir_lci", "sir_uci")})),
@@ -562,7 +656,7 @@ summarize_sir_results <- function(sir_df,
         yvar_name = data.table::first(yvar_name),
         group_observed = sum(observed, na.rm = TRUE),
         group_pyar = data.table::first(pyar),
-        group_n_base = data.table::first(group_n_base),
+        group_n_base = data.table::first(n_base),
         group_ref_inc_cases = sum(ref_inc_cases),
         group_ref_population_pyar = data.table::first(ref_population_pyar),
         group_expected = sum(expected, na.rm = TRUE),
@@ -578,6 +672,7 @@ summarize_sir_results <- function(sir_df,
       tidytable::rename.(observed = group_observed,
                          expected = group_expected,
                          pyar = group_pyar,
+                         n_base = group_n_base,
                          ref_inc_cases = group_ref_inc_cases,
                          ref_population_pyar = group_ref_population_pyar,
                          ref_inc_crude_rate = group_incidence_crude_rate) %>%
@@ -589,17 +684,32 @@ summarize_sir_results <- function(sir_df,
                                               if(collapse_ci == FALSE){c("sir_lci", "sir_uci")})),
                          tidyselect::everything()
       ) %>%
-      tidytable::mutate.(age = "Total",
-                         region = "Total",
-                         sex = "Total",
-                         year = "Total",
-                         t_site = "Total") %>%
+      tidytable::mutate.(age = paste0("Total - All included ages: ",  min_age, " - ", max_age),
+                         region = paste0("Total - All included regions: ", paste(used_region, collapse = ", ")),
+                         sex = paste0("Total - All included sexes: ", paste(used_sex, collapse = ", ")),
+                         year = paste0("Total - All included years: ", min_year, " - ", max_year),
+                         t_site = paste0("Total - All included Tumor sites: ", paste(used_t_site, collapse = ", "))
+      ) %>%
       tidytable::mutate_across.(.cols = c(pyar, sir, sir_lci, sir_uci), 
                                 .fns = ~ round(.x, 2))
-    
-    if(rs){
+    if(shorten_total_cols == TRUE){
       totals <- totals %>%
-        tidytable::mutate.(race = "Total")
+        tidytable::mutate.(age = "Total",
+                           region = "Total",
+                           sex = "Total",
+                           year = "Total",
+                           t_site = "Total")
+    }
+    
+    if(rs & shorten_total_cols == FALSE){
+      totals <- totals %>%
+        tidytable::mutate.(race = paste0("Total - All included races: ", paste(used_race, collapse = ", ")))
+      
+    } else{
+      if(rs & shorten_total_cols == TRUE){
+        totals <- totals %>%
+          tidytable::mutate.(race = "Total")
+      }
     }
     
     if(fu){
@@ -634,13 +744,24 @@ summarize_sir_results <- function(sir_df,
   ##--- enforce option output_information
   #full
   if(output_information == "full"){
-    sum_pre2 <- sum_pre
+    
+    sum_pre2 <- sum_pre %>%
+      tidytable::select.(tidyselect::any_of(c("age", "region", "sex", "year", if(rs){"race"}, 
+                                              if(yb){c("yvar_name", "yvar_label", "yvar_sort", "yvar_sort_levels")}, 
+                                              if(xb){c("xvar_name", "xvar_label")}, 
+                                              if(fu){c("fu_time", "fu_time_sort")}, 
+                                              "t_site", "observed", "expected", "sir",
+                                              if(collapse_ci == TRUE){"sir_ci"},
+                                              if(collapse_ci == FALSE){c("sir_lci", "sir_uci")},
+                                              "pyar", "n_base")
+      ),
+      tidyselect::everything())
   }
   #reduced
   if(output_information == "reduced"){
     sum_pre2 <- sum_pre %>%
       tidytable::select.(tidyselect::any_of(c("age", "region", "sex", "year", if(rs){"race"}, 
-                                              if(yb){c("yvar_name", "yvar_label")}, 
+                                              if(yb){c("yvar_name", "yvar_label", "yvar_sort", "yvar_sort_levels")}, 
                                               if(xb){c("xvar_name", "xvar_label")}, 
                                               if(fu){c("fu_time", "fu_time_sort")}, 
                                               "t_site", "observed", "expected", "sir",
@@ -742,7 +863,7 @@ summarize_sir_results <- function(sir_df,
       stringr::str_replace_all("_exp", "_2exp") %>% stringr::str_replace_all("_obs", "_1obs") %>% #add numer to facilitate sorting
       stringr::str_replace_all("_sir_uci", "_5sir_uci") %>% stringr::str_replace_all("_sir_lci", "_4sir_lci") %>%
       stringr::str_replace_all("_sir_ci", "_4sir_ci") %>% stringr::str_replace_all("_sir", "_3sir") %>% 
-      stringr::str_replace_all("_pyar", "_6pyar") %>% stringr::str_replace_all("_group_n_base", "_7group_n_base") %>%
+      stringr::str_replace_all("_pyar", "_6pyar") %>% stringr::str_replace_all("_n_base", "_7n_base") %>%
       stringr::str_sort() %>%
       stringr::str_replace_all(stringr::regex("\\_[:digit:]"), "_") 
     
@@ -793,3 +914,4 @@ summarize_sir_results <- function(sir_df,
   
 }
 
+
